@@ -968,7 +968,7 @@ testStandalone::testSmi_callingInterface()
 {
     STD_TEST_START(getTestName().c_str() << "  " );
 
-    std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI);
+    std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI_RAW);
     smi->setCommandIOMagic( 0x1234, 0x56 );
 
     smi::IDellCallingInterfaceSmi *ci = dynamic_cast<smi::IDellCallingInterfaceSmi *>(&*smi);
@@ -992,7 +992,7 @@ testStandalone::testSmi_callingInterface_physaddr ()
 {
     STD_TEST_START(getTestName().c_str() << "  " );
 
-    std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI);
+    std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI_RAW);
     smi->setCommandIOMagic( 0x1234, 0x56 );
 
     smi::IDellCallingInterfaceSmi *ci = dynamic_cast<smi::IDellCallingInterfaceSmi *>(&*smi);
