@@ -322,119 +322,20 @@ cout << "The BIOS Version is: " << item1->getString("BIOS Version") << endl;
          * outside the length of the present item.
          *
          * \param offset The offset to the field within the Item.
+         * \param out  void pointer to where to store output data
+         * \param size  size of data to return
+         *
          * \returns The (byte|word|dword|qword) at \a offset.  Throws 
          * smbios::SmbiosItemDataOutOfBounds or smbios::SmbiosParseException 
          * on error.
          *
+         *
          * \warning These methods are unchecked access. There is no verification
          * that (for example) when you use getU8() that the location you are 
          * trying to access is actually a U8. 
          */
-        virtual u8 getU8( unsigned int offset ) const = 0;
-        /** \copydoc getU8()  */
-        virtual u16 getU16( unsigned int offset ) const = 0; 
-        /** \copydoc getU8() */
-        virtual u32 getU32( unsigned int offset ) const = 0; 
-        /** \copydoc getU8() */
-        virtual u64 getU64( unsigned int offset ) const = 0;
 
-        /** Returns a byte field from the Item.
-         *
-         * This function takes a string that represents a FIELD element from
-         * the smbios XML definition. The implementation will look up the 
-         * offset in XML and call getU8( int ) with the correct offset.
-         *
-         * The implementation will also validate in the XML that the FIELD is 
-         * actually of length U8, and will throw an exception if the data field
-         * is the wrong length
-         *
-         * Sample XML:
-         * \verbatim
-        <STRUCTURE type="2" description="Base Board Information">
-            <FIELD offset="0h" name="Type" length="BYTE" usage="STRUCTURE_TYPE"/>
-            <FIELD offset="1h" name="Length" length="BYTE" usage="SIZE"/>
-            <FIELD offset="2h" name="Handle" length="WORD" usage="HANDLE"/>
-            <FIELD offset="4h" name="Manufacturer" length="BYTE" usage="STRING"/>
-            <FIELD offset="5h" name="Product" length="BYTE" usage="STRING"/>
-            <FIELD offset="6h" name="Version" length="BYTE" usage="STRING"/>
-            <FIELD offset="7h" name="Serial Number" length="BYTE" usage="STRING"/>
-        </STRUCTURE>
-         \endverbatim
-         * the \a field argument could specify, for example, "Manufacturer",
-         * and the NUMBER in offset 0x04 would be returned. The number in 
-         * this example represents the string number.
-         *
-         * \param field String corresponding to the FIELD element in the XML 
-         * \returns The byte in \a field.  Throws smbios::SmbiosItemDataOutOfBounds, 
-         * smbios::SmbiosParseException, or smbios::NotImplemented on error.
-         */
-        virtual u8 getU8( const std::string field ) const = 0;
-        /** \copydoc getU8( const std::string field ) const */
-        virtual u16 getU16( const std::string field ) const = 0; 
-        /** \copydoc getU8( const std::string field ) const */
-        virtual u32 getU32( const std::string field ) const = 0; 
-        /** \copydoc getU8( const std::string field ) const */
-        virtual u64 getU64( const std::string field ) const = 0; 
-
-
-        /** Returns the string at \a header_offset.  Offset is given as the field value
-         * for that particular string.  The string should almost always be retrieved with
-         * the getString(const std::string field) method instead. 
-         * \param header_offset The offset into the list of strings found at the end of the
-         * item.
-         * \returns The string if it is found.  Throws smbios::SmbiosItemDataOutOfBounds or
-         * smbios::InternalError on error.
-         */
-        virtual const char *getString(unsigned int header_offset) const = 0;
-
-        /** XML Enhanced access to string fields. 
-         * Returns the string for the field \a field.  Field must be of length "BYTE" and
-         * of usage "STRING". Type is checked against XML file. Other than that,
-         * this method is identical to getString(unsigned int):
-         *
-         * \param field A string representing the FIELD to look up. This is
-         * converted to the integer field offset by looking up in the XML file.
-         */
-        virtual const char *getString(const std::string field) const = 0;
-
-        /** Returns the bitfield specified by the \a offset into the item.
-         * \param offset The offset of the field within the item.
-         * \param fieldLen The total length of the bitfield.
-         * \param lsb The least significant bit of the bitfield.
-         * \param msb The most significant bit of the bitfield.  Optional.
-         * Default is the same as \a lsb if not supplied.
-         * \returns The bitfield will be contained in the lowest bits of the u32 upon return. 
-         * Throws smbios::SmbiosDataOutOfBounds on error.
-         *
-         * Sample XML:
-\verbatim
-        <FIELD offset="12h" name="BIOS Characteristics Extension Byte 1" length="BYTE" usage="BITFIELD">
-            <MAPPING>
-                <BITS lsb="0" name="ACPI supported"/> 
-                <BITS lsb="1" name="USB Legacy is supported"/> 
-                <BITS lsb="2" name="AGP is supported"/> 
-                <BITS lsb="3" name="I20 boot is supported"/> 
-                <BITS lsb="4" name="LS-120 boot is supported"/> 
-                <BITS lsb="5" name="ATAPI ZIP Drive boot is supported"/> 
-                <BITS lsb="6" name="1394 boot is supported"/> 
-                <BITS lsb="7" name="Smart Battery is supported"/> 
-            </MAPPING>
-        </FIELD>
-\endverbatim
-         *
-         * In this example, a call to getBitfield with an lsb of 2 and no msb would return the value of the 
-         * "AGP is supported" bit as the lowest bit of the u32 return value.
-         */
-        virtual u32 getBitfield( unsigned int offset, unsigned int fieldLen, unsigned int lsb, unsigned int msb=0 ) const = 0;
-
-        /** Returns the bitfield denoted by \a bitField.  Caller must be aware of the size of the bitfield.
-         * Bitfield will be returned in the lowest bits of the u32 return value.
-         * \param field The name of the field within the item.
-         * \param bitField The name of the bitfield to retrieve.
-         * \returns The value of the bitfield in the lowest bits of the u32.  Throws smbios::SmbiosParseException
-         * if error.
-         */
-        virtual u32 getBitfield( const std::string field, const std::string bitField ) const = 0;
+        virtual void getData( unsigned int offset, void *out, size_t size ) const = 0;
 
         //loathe Stroustrup. In his infinite wisdom, he asserts that
         //auto_ptr equivalent for array is _not_necessary_.
@@ -457,9 +358,28 @@ cout << "The BIOS Version is: " << item1->getString("BIOS Version") << endl;
             FIELD_LEN_DWORD=4,
             FIELD_LEN_QWORD=8
         };
-
     };
 
+    u8 getItemType(const ISmbiosItem &item);
+    u8 getItemLength(const ISmbiosItem &item);
+    u16 getItemHandle(const ISmbiosItem &item);
+
+    u8 getU8_FromItem(const ISmbiosItem &item, unsigned int offset);
+    u16 getU16_FromItem(const ISmbiosItem &item, unsigned int offset);
+    u32 getU32_FromItem(const ISmbiosItem &item, unsigned int offset);
+    u64 getU64_FromItem(const ISmbiosItem &item, unsigned int offset);
+    const char *getString_FromItem(const ISmbiosItem &item, unsigned int offset);
+    void *getBits_FromItem(const ISmbiosItem &item, unsigned int offset, void *out, unsigned int lsb=0, unsigned int msb=0 );
+    bool isBitSet(const ISmbiosItem *itemPtr, unsigned int offset, unsigned int bitToTest);
+
+    template <class R> 
+    R &getData(const ISmbiosItem &item, unsigned int offset, R &out)
+    {
+        item.getData(offset, &out, sizeof(R));
+        return out;
+    }
+
+
     //! Iterator base class for ISmbiosTable objects.
     /** The base class for iterators over ISmbiosTable. This class has all of
      * the data items to keep track of the position. There is no good way to
