@@ -247,6 +247,10 @@ namespace smbios
     std::ostream & operator << (std::ostream & cout, const ITokenTable & item);
     std::ostream & operator << (std::ostream & cout, const IToken & item);
 
+    // helper functions
+
+    bool isTokenActive(int tokenNum);
+    void activateToken(int tokenNum, std::string password = "");
 }
 
 // always should be last thing in header file
