@@ -312,7 +312,7 @@ namespace smbios
         // for broken BIOSen.
         // The (3) is to take into account the deref at the end "data[0] ||
         // data[1]", and to take into account the "data += 2" on the next line.
-        while (((data - smbiosBuffer) < (table_header.table_length - 3)) && (*data || data[1]))
+        while (((data - smbiosBuffer) < (table_header.dmi.table_length - 3)) && (*data || data[1]))
             data++;
 
         // ok, skip past the actual double null.
@@ -321,7 +321,7 @@ namespace smbios
         // add code specifically to work around crap bios implementations
         // that do not have the _required_ 0x7f end-of-table entry
         //   note: (4) == sizeof a std header.
-        if ( (data - smbiosBuffer) > (table_header.table_length - 4))
+        if ( (data - smbiosBuffer) > (table_header.dmi.table_length - 4))
         {
             // really should output some nasty message here... This is very
             // broken
@@ -336,7 +336,7 @@ out1:
 
     int SmbiosTable::getNumberOfEntries () const
     {
-        return table_header.table_num_structs;
+        return table_header.dmi.table_num_structs;
     }
 
     smbiosLowlevel::smbios_table_entry_point SmbiosTable::getTableEPS() const
@@ -351,10 +351,10 @@ out1:
         cout << static_cast<int>(table_header.major_ver) << ".";
         cout << static_cast<int>(table_header.minor_ver) << endl;
         cout << hex ;
-        cout << "\taddress    : " << table_header.table_address << endl;
+        cout << "\taddress    : " << table_header.dmi.table_address << endl;
         cout << dec;
-        cout << "\tlength     : " << table_header.table_length << endl;
-        cout << "\tnum structs: " << table_header.table_num_structs << endl;
+        cout << "\tlength     : " << table_header.dmi.table_length << endl;
+        cout << "\tnum structs: " << table_header.dmi.table_num_structs << endl;
         cout << endl;
 
         SmbiosTable::const_iterator position = begin();
