@@ -42,6 +42,8 @@ using namespace std;
 
 namespace smi
 {
+
+
     //
     // ISmi functions
     //
@@ -136,6 +138,12 @@ namespace smi
 
         smiStrategy->finish();
 
+        if( -6 == smi_buf.cbRES1 )
+            throw SmiExceptionImpl("Output buffer not large enough.");
+
+        if( -5 == smi_buf.cbRES1 )
+            throw SmiExceptionImpl("Output buffer format error.");
+
         if( -3 == smi_buf.cbRES1 )
             throw UnhandledSmiImpl("Unhandled SMI call.");
 
@@ -195,31 +203,6 @@ namespace smi
         std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI);
         smi::IDellCallingInterfaceSmi *ci = dynamic_cast<smi::IDellCallingInterfaceSmi *>(smi.get());
 
-        size_t len = 0;
-        const u8 *buf = 0;
-        try
-        {
-            buf = (*table)[0xDA]->getBufferCopy(len);
-            // have to take off the size of the optional structs or this is a false positive 
-            // on some systems
-            if( len < (sizeof(smbios::calling_interface_structure) - sizeof(smbios::calling_interface_token)) )
-                throw smbios::InternalErrorImpl("Internal error. Possible BIOS problem: length of 0xDA structure is too short.");
-
-            const smbios::calling_interface_structure *struct_ptr = reinterpret_cast<const smbios::calling_interface_structure*>(buf);
-            smi->setCommandIOMagic( struct_ptr->cmdIOAddress, struct_ptr->cmdIOCode );
-            delete [] const_cast<u8 *>(buf);
-        }
-        catch(const smbios::DerefNullPointer &)
-        {
-            delete [] const_cast<u8 *>(buf);
-            throw smbios::InternalErrorImpl("Internal error. It does not appear that this BIOS supports SMI (no 0xDA structure in SMBIOS).");
-        }
-        catch(...)
-        {
-            delete [] const_cast<u8 *>(buf);
-            throw;
-        }
-
         ci->setClass( smiClass );
         ci->setSelect( select );
         ci->setArg(0, args[0]);
@@ -622,4 +605,204 @@ out:
         }
     }
 
+
+    // token list for wireless tokens. Probably not the best place for this,
+    // but until something better comes along...
+    const int Bluetooth_Devices_Disable = 0x0153;  // docs appear to be wrong. They say 0x0152, but this looks backwards from reality
+    const int Bluetooth_Devices_Enable = 0x0152;  // docs appear to be wrong. They say 0x0153, but this looks backwards from reality
+    const int Cellular_Radio_Disable = 0x017B;
+    const int Cellular_Radio_Enable = 0x017C;
+    const int WiFi_Locator_Disable = 0x017D;
+    const int WiFi_Locator_Enable = 0x017E;
+    const int Wireless_LAN_Disable = 0x017F;
+    const int Wireless_LAN_Enable = 0x0180;
+    const int Wireless_Switch_Bluetooth_Control_Disable = 0x0181;
+    const int Wireless_Switch_Bluetooth_Control_Enable = 0x0182;
+    const int Wireless_Switch_Cellular_Control_Disable = 0x0183;
+    const int Wireless_Switch_Cellular_Control_Enable = 0x0184;
+    const int Wireless_Switch_Wireless_LAN_Control_Disable = 0x0185;
+    const int Wireless_Switch_Wireless_LAN_Control_Enable = 0x0186;
+    const int Radio_Transmission_Enable = 0x010c;
+    const int Radio_Transmission_Disable = 0x010d;
+    const int Wireless_Device_Disable = 0x0114;
+    const int Wireless_Device_App_Control = 0x0115;
+    const int Wireless_Device_App_Or_Hotkey_Control = 0x0116;
+
+    // cbClass 17
+    // cbSelect 11
+    // WiFi Control
+    // Entry/return values grouped by the value of cbARG1, byte0 which indicates the function to perform:
+    // 0x1 = Set QuickSet Radio Disable Flag
+    //  cbARG1, byte1    Radio ID value:
+    //      0 Radio Status
+    //      1 WLAN ID
+    //      2 BT ID
+    //      3 WWAN ID
+    //  cbARG1, byte2   Flag bits:
+    //      0 QuickSet disables radio (1)
+    //      1-7 Reserved (0)
+    //  cbRES1      Standard return codes (0, -1, -2)
+    //  cbRES2      QuickSet (QS) radio disable bit map:
+    //      0 QS disables WLAN
+    //      1 QS disables BT
+    //      2 QS disables WWAN
+    //      3-31 Reserved (0)
+    void wirelessRadioControl(bool enable, bool boot, bool runtime, int enable_token, int disable_token, int radioNum, std::string password)
+    {
+        if (boot)
+            smbios::activateToken( (enable ?
+                enable_token :
+                disable_token),
+                password
+                );
+        if (runtime)
+        {
+            if (enable && !smbios::isTokenActive(enable_token))
+                throw ConfigErrorImpl("boot time config disabled, runtime setting has no effect.");
+
+            u32 disable = enable ? 0:1;
+            u32 args[4] = {(1 | (static_cast<u32>(radioNum)<<8) | ((disable)<<16)), 0, 0, 0};
+            u32 res[4] = {0,};
+            doSimpleCallingInterfaceSmi(17, 11, args, res);
+        }
+    }
+
+
+    static void switchControl(u32 whichConfig, u32 whichSwitch, bool enable)
+    {
+        std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI);
+        smi::IDellCallingInterfaceSmi *ci = dynamic_cast<smi::IDellCallingInterfaceSmi *>(smi.get());
+        ci->setClass( 17 );  /* ? */
+        ci->setSelect( 11 );  /* WiFi Control */
+    
+        // 0x2 = Wireless Switch Configuration
+        //  cbARG1, byte1   Subcommand:
+        //      0 Get config
+        //      1 Set config
+        //      2 Set WiFi locator enable/disable
+        //  cbARG1,byte2    Switch settings (if byte 1==1):
+        //      0 WLAN switch control (1)
+        //      1 BT switch control (1)
+        //      2 WWAN switch control (1)
+        //      3-7 Reserved (0)
+        //  cbARG1, byte2   Enable bits (if byte 1==2):
+        //      0 Enable WiFi locator (1)
+        //  cbRES1      Standard return codes (0, -1, -2)
+        //  cbRES2      QuickSet radio disable bit map:
+        //      0 WLAN controlled by switch (1)
+        //      1 BT controlled by switch (1)
+        //      2 WWAN controlled by switch (1)
+        //      3-6 Reserved (0)
+        //      7 Wireless switch config locked (1)
+        //      8 WiFi locator enabled (1)
+        //      9-14 Reserved (0)
+        //      15 WiFi locator setting locked (1)
+        //      16-31 Reserved (0)
+        ci->setArg(smi::cbARG1, 0x2);
+        ci->execute();
+    
+        u32 oldConfig = ci->getRes(smi::cbRES2);
+        if (whichConfig == 1)
+            oldConfig &= 0xFF;
+        else if (whichConfig == 2)
+            oldConfig = ((oldConfig>>8) & 0xFF);
+    
+        u32 newConfig = (oldConfig & ~whichSwitch) | ((enable?1:0) * whichSwitch);
+        ci->setArg(smi::cbARG1, (0x2 | (whichConfig << 8) | (newConfig << 16)));
+        ci->execute();
+    }
+    
+    void wirelessSwitchControl(bool enable, bool boot, bool runtime, int enable_token, int disable_token, int switchNum, std::string password)
+    {
+        int intSwitchConfig = 0, intSwitchNum = 0;
+        switch(switchNum)
+        {
+            case WLAN_SWITCH_CTL:
+                intSwitchConfig = 1;
+                intSwitchNum = 1;
+                break;
+            case BLUETOOTH_SWITCH_CTL:
+                intSwitchConfig = 1;
+                intSwitchNum = 2;
+                break;
+            case WWAN_SWITCH_CTL:
+                intSwitchConfig = 1;
+                intSwitchNum = 4;
+                break;
+            case LOCATOR_SWITCH_CTL:
+                intSwitchConfig = 2;
+                intSwitchNum = 1;
+                break;
+            default:
+                throw ParameterErrorImpl("Invalid switch number passed to wirelessSwitchControl()");
+        }
+        
+        if (boot)
+            smbios::activateToken( (enable ?
+                enable_token :
+                disable_token),
+                password
+                );
+        if (runtime)
+            switchControl(intSwitchConfig, intSwitchNum, enable);
+    }
+
+    radioStatusCode wirelessRadioStatus(radioNum which, std::ostream &cout, u32 defRes2)
+    {
+        radioStatusCode ret = STATUS_UNKNOWN;
+        try
+        {
+            u32 args[4] = {0,}, res[4] ={0,};
+            if (!defRes2)
+                smi::doSimpleCallingInterfaceSmi(17, 11, args, res);
+            else
+                res[smi::cbRES2] = defRes2;
+    
+            int supported_bit=0, installed_bit=0, disabled_bit=0;
+            string name;
+            switch(which)
+            {
+                case smi::WLAN_RADIO_NUM:
+                    supported_bit = 2;
+                    installed_bit = 8;
+                    disabled_bit = 17;
+                    name = "WLAN";
+                    break;
+                case smi::BLUETOOTH_RADIO_NUM:
+                    supported_bit = 3;
+                    installed_bit = 9;
+                    disabled_bit = 18;
+                    name = "Bluetooth";
+                    break;
+                case smi::WWAN_RADIO_NUM:
+                    supported_bit = 4;
+                    installed_bit = 10;
+                    disabled_bit = 19;
+                    name = "WWAN";
+                    break;
+            }
+    
+            cout << "Radio Status for " << name << ":" << endl;
+            if (res[smi::cbRES2] & (1 << supported_bit))
+            {
+                cout << "\t" << name << " supported" << endl;
+                cout << "\t" << name << " " << ((res[smi::cbRES2] & (1 << installed_bit)) ? "installed":"not installed") << endl;
+                cout << "\t" << name << " " << ((res[smi::cbRES2] & (1 << disabled_bit)) ? "disabled" : "enabled") << endl;
+    
+                ret = STATUS_DISABLED;
+                if (!(res[smi::cbRES2] & (1 << installed_bit)))
+                    ret = STATUS_NOT_PRESENT;
+                else if (!(res[smi::cbRES2] & (1 << disabled_bit)))
+                    ret = STATUS_ENABLED;
+            } else {
+                cout << "\t" << name << " not supported" << endl;
+                ret = STATUS_UNSUPPORTED;
+        }
+            cout << "\tStatus Code: " << ret << endl;
+        } catch (smi::UnsupportedSmi &) {
+            // this interface not available...
+        }
+        return ret;
+    }
+
 }
