@@ -19,6 +19,7 @@
 #define LIBSMBIOS_SOURCE
 #include "SmiImpl.h"
 #include "FactoryImpl2.h"
+#include "TokenImpl.h"
 
 // message.h should be included last.
 #include "smbios/message.h"
@@ -63,10 +64,47 @@ namespace smi
             strategyPtr = new SmiMockStrategy(getParameterString("smiFile"));
 
 
+        // used for automatic setup of magic io stuff 
+        u16 cmdIOAddress = 0;
+        u8  cmdIOCode = 0;
+        smbios::TokenTableFactory *ttFactory = 0;
+        smbios::ITokenTable *tokenTable = 0;
+
         switch( type )
         {
+        case DELL_CALLING_INTERFACE_SMI_RAW:
+            ret = new DellCallingInterfaceSmiImpl(strategyPtr);
+            break;
+
         case DELL_CALLING_INTERFACE_SMI:
             ret = new DellCallingInterfaceSmiImpl(strategyPtr);
+            // automatically set up cmd io port/magic
+            // step 1: get token table
+            ttFactory = smbios::TokenTableFactory::getFactory() ;
+            tokenTable = ttFactory->getSingleton();
+            // step 2: iterate through token table
+            for(
+                smbios::ITokenTable::iterator token = tokenTable->begin();
+                token != tokenTable->end();
+                ++token )
+            {
+                // Step 3: go until we get to the first one that will dynamic cast 
+                //         into a TokenDA
+                try{
+                    if( token->getTokenClass() != "TokenDA" )
+                        continue;
+
+                    // Step 4: then set cmd io stuff
+                    dynamic_cast<smbios::ISmiToken *>(&*token)->getSmiDetails(&cmdIOAddress, &cmdIOCode, static_cast<u8*>(0));
+                    ret->setCommandIOMagic( cmdIOAddress, cmdIOCode );
+                    break;
+                } catch(...){
+                }
+            }
+
+            if( ! (cmdIOAddress && cmdIOCode))
+                throw SmiExceptionImpl(_("Could not automatically setup up magic io"));
+
             break;
         default:
             throw InvalidSmiModeImpl(_("Unknown smi factory mode requested"));
