@@ -24,8 +24,8 @@
 #include "smbios/SystemInfo.h"
 #include "smbios/IMemory.h"
 #include "smbios/SmbiosDefs.h"
-#include "../common/ExceptionImpl.h"
-#include "../token/TokenLowLevel.h"
+#include "ExceptionImpl.h"
+#include "TokenLowLevel.h"
 
 #include "DellMagic.h"
 
@@ -244,8 +244,7 @@ static char *getStringFromTable(unsigned int structure, unsigned int stringNumbe
         throw InternalErrorImpl();
 
     const char *tempval = 0;
-    smbios::ISmbiosTable::iterator item;
-    tempval = (*table)[structure]->getString(stringNumber);
+    tempval = getString_FromItem(*(*table)[structure], stringNumber);
 
     if(!tempval)
         throw exception();
@@ -499,7 +498,9 @@ static char *getAssetTagFromSysEncl()
     return getStringFromTable(System_Enclosure_or_Chassis, System_Enclosure_or_Chassis_Asset_Offset);
 }
 
-static char *getAssetTagFromToken()
+// not static so we can use it in unit test, but not part of public API.
+// you have been warned.
+char *getAssetTagFromToken()
 {
     smbios::ITokenTable *table = 0;
     table = smbios::TokenTableFactory::getFactory()->getSingleton();
