@@ -48,4 +48,24 @@ namespace smbios
     {
         return item.streamify(cout);
     }
+
+    void activateToken(int tokenNum, string password)
+    {
+        smbios::TokenTableFactory *ttFactory = smbios::TokenTableFactory::getFactory() ;
+        smbios::ITokenTable *tokenTable = ttFactory->getSingleton();
+        try
+        {
+            smbios::IToken *token = &(*((*tokenTable)[ tokenNum ]));
+            dynamic_cast< smbios::IProtectedToken * >(token)->tryPassword(password);
+        } catch (...) { /* not an error if password fails */ }
+
+        (*tokenTable)[ tokenNum ]->activate();
+    }   
+    
+    bool isTokenActive(int token)
+    {   
+        smbios::TokenTableFactory *ttFactory = smbios::TokenTableFactory::getFactory() ;
+        smbios::ITokenTable *tokenTable = ttFactory->getSingleton();
+        return (*tokenTable)[ token ]->isActive();
+    } 
 }
