@@ -30,6 +30,14 @@
 
 using namespace std;
 
+#if defined(DEBUG_TOKEN_DA)
+#   define DCOUT(line) do { cout << line; } while(0)
+#   define DCERR(line) do { cerr << line; } while(0)
+#else
+#   define DCOUT(line) do {} while(0)
+#   define DCERR(line) do {} while(0)
+#endif
+
 namespace smbios
 {
     SmiTokenDA::SmiTokenDA( const smbios::ISmbiosItem &initItem, const calling_interface_token *initToken )
@@ -88,16 +96,8 @@ namespace smbios
     {
         bool ret = false;
 
-        std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI);
-        smi->setCommandIOMagic( structure.cmdIOAddress, structure.cmdIOCode );
-
-        smi::IDellCallingInterfaceSmi *ci = dynamic_cast<smi::IDellCallingInterfaceSmi *>(smi.get());
-        ci->setClass( 0x0 );  /* Read Non-Volatile Storage class code */
-        ci->setSelect( 0x0 );  /* Read Non-Volatile Storage select code */
-        ci->setArg( 0, token.location );
-        ci->execute();
-
-        if (ci->getRes(2) == token.value )
+        DCERR("reading token: " << static_cast<u32>(token.location) << "  compare with value: " << static_cast<u32>(token.value) << "  actual: " << smi::readNVStorage(token.location, 0, 0) << endl);
+        if (token.value == smi::readNVStorage(token.location, 0, 0))
             ret = true;
 
         return ret;
@@ -126,14 +126,8 @@ namespace smbios
 
     void SmiTokenDA::activate() const
     {
-        std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI);
-        smi->setCommandIOMagic( structure.cmdIOAddress, structure.cmdIOCode );
-        smi::IDellCallingInterfaceSmi *ci = dynamic_cast<smi::IDellCallingInterfaceSmi *>(smi.get());
-        ci->setClass( 0x1 );  /* Write Non-Volatile Storage class code */
-        ci->setSelect( 0x0 );  /* Write Non-Volatile Storage select code */
-        ci->setArg( 0, token.location );
-        ci->setArg( 1, token.value );
-        executeWithPassword(ci, 2, password);
+        DCERR("trying to activate token: " << static_cast<u32>(token.location) << "  with value: " << static_cast<u32>(token.value) << endl);
+        smi::writeNVStorage(password, token.location, token.value, 0, 0);
     }
 
     bool SmiTokenDA::isString() const
@@ -148,7 +142,7 @@ namespace smbios
 
     const string SmiTokenDA::getString(u8 *byteArray, unsigned int size ) const
     {
-        std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI);
+        std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI_RAW);
         smi->setCommandIOMagic( structure.cmdIOAddress, structure.cmdIOCode );
 
         smi::IDellCallingInterfaceSmi *ci = dynamic_cast<smi::IDellCallingInterfaceSmi *>(smi.get());
@@ -177,7 +171,7 @@ namespace smbios
         if( size < 2 )
             return;
 
-        std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI);
+        std::auto_ptr<smi::ISmi> smi = smi::SmiFactory::getFactory()->makeNew(smi::SmiFactory::DELL_CALLING_INTERFACE_SMI_RAW);
         smi->setCommandIOMagic( structure.cmdIOAddress, structure.cmdIOCode );
 
         smi::IDellCallingInterfaceSmi *ci = dynamic_cast<smi::IDellCallingInterfaceSmi *>(smi.get());
